Package: UpSetR
Title: A More Scalable Alternative to Venn and Euler Diagrams for
        Visualizing Intersecting Sets
Version: 1.4.1
Date: 2026-5-20
URL: https://github.com/hms-dbmi/UpSetR
BugReports: https://github.com/hms-dbmi/UpSetR/issues
Authors@R: c(
    person("Jake", "Conway", email = "jake.conway@pathai.com", role = "aut"),
    person("Nils", "Gehlenborg", email = "nils@hms.harvard.edu", role= c("aut","cre")))
Description: Creates visualizations of intersecting sets using a novel matrix
    design, along with visualizations of several common set, element and attribute
    related tasks (Conway 2017) <doi:10.1093/bioinformatics/btx364>.
Depends: R (>= 3.0)
Imports: ggplot2, gridExtra, plyr, utils, stats, methods, grDevices,
        scales
License: MIT + file LICENSE
VignetteBuilder: knitr
Suggests: knitr
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-20 20:37:51 UTC; nils
Author: Jake Conway [aut],
  Nils Gehlenborg [aut, cre]
Maintainer: Nils Gehlenborg <nils@hms.harvard.edu>
Repository: CRAN
Date/Publication: 2026-05-25 16:10:02 UTC
Built: R 4.6.0; ; 2026-05-25 17:44:18 UTC; unix
