# Generated by roxygen2: do not edit by hand

S3method(print,upset)
S3method(summary,upset)
export(elements)
export(fromExpression)
export(fromList)
export(histogram)
export(intersects)
export(scatter_plot)
export(upset)
import(ggplot2)
import(grDevices)
import(gridExtra)
import(methods)
import(scales)
import(stats)
import(utils)
importFrom(grid,gpar)
importFrom(grid,grid.draw)
importFrom(grid,grid.layout)
importFrom(grid,grid.newpage)
importFrom(grid,legendGrob)
importFrom(grid,popViewport)
importFrom(grid,pushViewport)
importFrom(grid,viewport)
importFrom(plyr,count)
