% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/elements.R
\name{elements}
\alias{elements}
\title{Element query for queries parameter}
\usage{
elements(func, query, ...)
}
\arguments{
\item{func}{A functions provided internally}

\item{query}{Input from the queries parameter assigned to the element function.}

\item{...}{Additional parameters to be supplied internally}
}
\description{
A query paramter to visualize specific elements of interest if queries = active (in custom plots only)
}
\note{
See examples section of upset function on how to use this function in the queries parameter.
}
