% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fromList.R
\name{fromList}
\alias{fromList}
\title{List of named vectors to UpSetR converter}
\usage{
fromList(input)
}
\arguments{
\item{input}{A list of named vectors to be converted to a data frame compatible with UpSetR}
}
\description{
A function to convert a list of named vectors to a data frame compatible with UpSetR.
}
\note{
See "Basic Usage" vignette for an example on how to use this function in UpSetR.
}
