% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fromExpression.R
\name{fromExpression}
\alias{fromExpression}
\title{Expression to UpSetR converters}
\usage{
fromExpression(input)
}
\arguments{
\item{input}{An vector (expression) to be converted to an input compatible with UpSetR}
}
\description{
A function to convert an expression to a data frame compatible with UpSetR.
}
\note{
See "Basic Usage" vignette for an example on how to use this function in UpSetR.
}
