% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scatter_plot.R
\name{scatter_plot}
\alias{scatter_plot}
\title{Scatterplot for customplot}
\usage{
scatter_plot(mydata, x, y)
}
\arguments{
\item{mydata}{A data set containing intersection data provided internally}

\item{x}{The x aesthetic for the scatter plot}

\item{y}{The y aesthetic for the scatter plot}
}
\description{
A pre-made scatter plot that can be added to the custom.plot parameter.
}
\note{
See examples section for upset function on how to use custom.plot parameter.
}
