% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/histogram.R
\name{histogram}
\alias{histogram}
\title{Histogram for custom plot}
\usage{
histogram(mydata, x)
}
\arguments{
\item{mydata}{A data set containing intersection data provided internally}

\item{x}{The x aesthetic of for the histogram plot}
}
\description{
A pre-made histogram that can be added to custom.plot parameter.
}
\note{
See examples section for upset function on how to use custom.plot parameter
}
