% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/intersects.R
\name{intersects}
\alias{intersects}
\title{Intersection query for queries parameter}
\usage{
intersects(func, query, ...)
}
\arguments{
\item{func}{A functions provided internally}

\item{query}{Input from the queries parameter assigned to the intersection function.}

\item{...}{Additional parameters to be applied internally}
}
\description{
A query paramter to visualize elements contained in specific intersections
}
\note{
See examples section of upset function on how to use this function in the queries parameter.
}
